@@ -42,6 +42,7 @@
 #define HAVE_LIMITS_H 1
 #define HAVE_LINK_H 1
 /* #undef HAVE_LINUX_UNISTD_H */
+/* #undef HAVE_LINUX_MMAN_H */
 #define HAVE_LOCALE_H 1
 /* #undef HAVE_MALLOC_H */
 #define HAVE_MEMORY_H 1
@@ -272,10 +273,10 @@
   #define SIZEOF_SIZE_T  SIZEOF_LONG
 #else
 /* No indentation, to fetch the lines from verification scripts */
-#define SIZEOF_LONG   4
-#define SIZEOF_VOIDP  4
-#define SIZEOF_CHARP  4
-#define SIZEOF_SIZE_T 4
+#define SIZEOF_LONG   8
+#define SIZEOF_VOIDP  8
+#define SIZEOF_CHARP  8
+#define SIZEOF_SIZE_T 8
 #endif
 
 #define HAVE_LONG 1
@@ -290,7 +291,7 @@
 /* #undef HAVE_UCHAR */
 #define SIZEOF_UINT 4
 #define HAVE_UINT 1
-#define SIZEOF_ULONG 4
+#define SIZEOF_ULONG 8
 #define HAVE_ULONG 1
 /* #undef SIZEOF_INT8 */
 /* #undef HAVE_INT8 */
@@ -328,7 +329,7 @@
     #define WORDS_BIGENDIAN 1
   #endif
 #else
-/* #undef WORDS_BIGENDIAN */
+#define WORDS_BIGENDIAN 1 
 #endif
 
 /* Define to `__inline__' or `__inline' if that's what the C compiler calls
@@ -389,8 +390,8 @@
 #define STACK_DIRECTION -1
 
 #define SYSTEM_TYPE "OpenBSD"
-#define MACHINE_TYPE "i386"
-#define DEFAULT_MACHINE "i386"
+#define MACHINE_TYPE "powerpc64"
+#define DEFAULT_MACHINE "powerpc64"
 /* #undef HAVE_DTRACE */
 
 #define SIGNAL_WITH_VIO_CLOSE 1
@@ -525,16 +526,16 @@
 
 #define MYSQL_VERSION_MAJOR 10
 #define MYSQL_VERSION_MINOR 5
-#define MYSQL_VERSION_PATCH 8
+#define MYSQL_VERSION_PATCH 9
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.5.8"
+#define PACKAGE_STRING "MySQL Server 10.5.9"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.5.8"
-#define VERSION "10.5.8"
+#define PACKAGE_VERSION "10.5.9"
+#define VERSION "10.5.9"
 #define PROTOCOL_VERSION 10
 #define PCRE2_CODE_UNIT_WIDTH 8
 
