@@ -1,5 +1,6 @@
 /* -*- C++ -*- */
 /* Copyright (c) 2002, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2009, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -46,18 +47,18 @@ template <typename T> class SQL_I_List;
 /*
   Values for the type enum. This reflects the order of the enum declaration
   in the CREATE TABLE command.
+  See also storage/perfschema/my_thread.h
 */
-enum stored_procedure_type
+enum enum_sp_type
 {
-  TYPE_ENUM_FUNCTION=1,
-  TYPE_ENUM_PROCEDURE=2,
-  TYPE_ENUM_PACKAGE=3,
-  TYPE_ENUM_PACKAGE_BODY=4,
-  TYPE_ENUM_TRIGGER=5,
-  TYPE_ENUM_PROXY=6
+  SP_TYPE_FUNCTION=1,
+  SP_TYPE_PROCEDURE=2,
+  SP_TYPE_PACKAGE=3,
+  SP_TYPE_PACKAGE_BODY=4,
+  SP_TYPE_TRIGGER=5,
+  SP_TYPE_EVENT=6,
 };
 
-
 class Sp_handler
 {
   bool sp_resolve_package_routine_explicit(THD *thd,
@@ -120,13 +121,13 @@ public: // TODO: make it private or protected
 public:
   virtual ~Sp_handler() {}
   static const Sp_handler *handler(enum enum_sql_command cmd);
-  static const Sp_handler *handler(stored_procedure_type type);
+  static const Sp_handler *handler(enum_sp_type type);
   static const Sp_handler *handler(MDL_key::enum_mdl_namespace ns);
   /*
     Return a handler only those SP objects that store
     definitions in the mysql.proc system table
   */
-  static const Sp_handler *handler_mysql_proc(stored_procedure_type type)
+  static const Sp_handler *handler_mysql_proc(enum_sp_type type)
   {
     const Sp_handler *sph= handler(type);
     return sph ? sph->sp_handler_mysql_proc() : NULL;
@@ -135,9 +136,8 @@ public:
   static bool eq_routine_name(const LEX_CSTRING &name1,
                               const LEX_CSTRING &name2)
   {
-    return my_strnncoll(system_charset_info,
-                        (const uchar *) name1.str, name1.length,
-                        (const uchar *) name2.str, name2.length) == 0;
+    return system_charset_info->strnncoll(name1.str, name1.length,
+                                          name2.str, name2.length) == 0;
   }
   const char *type_str() const { return type_lex_cstring().str; }
   virtual const char *show_create_routine_col1_caption() const
@@ -154,7 +154,7 @@ public:
   {
     return this;
   }
-  virtual stored_procedure_type type() const= 0;
+  virtual enum_sp_type type() const= 0;
   virtual LEX_CSTRING type_lex_cstring() const= 0;
   virtual LEX_CSTRING empty_body_lex_cstring(sql_mode_t mode) const
   {
@@ -249,7 +249,7 @@ public:
 class Sp_handler_procedure: public Sp_handler
 {
 public:
-  stored_procedure_type type() const { return TYPE_ENUM_PROCEDURE; }
+  enum_sp_type type() const { return SP_TYPE_PROCEDURE; }
   LEX_CSTRING type_lex_cstring() const
   {
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("PROCEDURE")};
@@ -299,7 +299,7 @@ public:
 class Sp_handler_function: public Sp_handler
 {
 public:
-  stored_procedure_type type() const { return TYPE_ENUM_FUNCTION; }
+  enum_sp_type type() const { return SP_TYPE_FUNCTION; }
   LEX_CSTRING type_lex_cstring() const
   {
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("FUNCTION")};
@@ -368,7 +368,7 @@ public: // TODO: make it private or protected
                                const Database_qualified_name *name)
                                const;
 public:
-  stored_procedure_type type() const { return TYPE_ENUM_PACKAGE; }
+  enum_sp_type type() const { return SP_TYPE_PACKAGE; }
   LEX_CSTRING type_lex_cstring() const
   {
     static LEX_CSTRING m_type_str= {STRING_WITH_LEN("PACKAGE")};
@@ -401,7 +401,7 @@ public:
 class Sp_handler_package_body: public Sp_handler_package
 {
 public:
-  stored_procedure_type type() const { return TYPE_ENUM_PACKAGE_BODY; }
+  enum_sp_type type() const { return SP_TYPE_PACKAGE_BODY; }
   LEX_CSTRING type_lex_cstring() const
   {
     static LEX_CSTRING m_type_str= {STRING_WITH_LEN("PACKAGE BODY")};
@@ -434,7 +434,7 @@ public:
 class Sp_handler_trigger: public Sp_handler
 {
 public:
-  stored_procedure_type type() const { return TYPE_ENUM_TRIGGER; }
+  enum_sp_type type() const { return SP_TYPE_TRIGGER; }
   LEX_CSTRING type_lex_cstring() const
   {
     static LEX_CSTRING m_type_str= { STRING_WITH_LEN("TRIGGER")};
@@ -493,20 +493,20 @@ inline const Sp_handler *Sp_handler::handler(enum_sql_command cmd)
 }
 
 
-inline const Sp_handler *Sp_handler::handler(stored_procedure_type type)
+inline const Sp_handler *Sp_handler::handler(enum_sp_type type)
 {
   switch (type) {
-  case TYPE_ENUM_PROCEDURE:
+  case SP_TYPE_PROCEDURE:
     return &sp_handler_procedure;
-  case TYPE_ENUM_FUNCTION:
+  case SP_TYPE_FUNCTION:
     return &sp_handler_function;
-  case TYPE_ENUM_PACKAGE:
+  case SP_TYPE_PACKAGE:
     return &sp_handler_package_spec;
-  case TYPE_ENUM_PACKAGE_BODY:
+  case SP_TYPE_PACKAGE_BODY:
     return &sp_handler_package_body;
-  case TYPE_ENUM_TRIGGER:
+  case SP_TYPE_TRIGGER:
     return &sp_handler_trigger;
-  case TYPE_ENUM_PROXY:
+  case SP_TYPE_EVENT:
     break;
   }
   return NULL;
@@ -655,7 +655,7 @@ extern "C" uchar* sp_sroutine_key(const uchar *ptr, size_t *plen,
   Routines which allow open/lock and close mysql.proc table even when
   we already have some tables open and locked.
 */
-TABLE *open_proc_table_for_read(THD *thd, Open_tables_backup *backup);
+TABLE *open_proc_table_for_read(THD *thd);
 
 bool load_charset(MEM_ROOT *mem_root,
                   Field *field,
